﻿
Public Class RoboTurretDemo

    Dim port As New System.IO.Ports.SerialPort
    Dim bytepacketsend As New Dictionary(Of String, Byte)
    Dim pantilt1023 As New Dictionary(Of String, Integer)
    Dim RR As New RR_COM_APILib.API_Wrapper

    Dim intCameraPixelWidth As Integer
    Dim intCameraPixelHeight As Integer
    Dim intCenterPan As Integer
    Dim intCenterTilt As Integer
    Dim intOffsetPan As Integer
    Dim intOffsetTilt As Integer
    Dim intTempPan As Integer
    Dim intTempTilt As Integer
    Dim cogX As Integer
    Dim cogY As Integer
    Dim intBlobSizeMinimum As Integer
    Dim intFactor As Integer


#Region "Form Open & Close"

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        'Populate the com port drop down
        For Each port As String In System.IO.Ports.SerialPort.GetPortNames()
            cmbPorts.Items.Add(port)
            cmbPorts.Text = port
        Next

        'populate our listbox
        lstColor.Items.Add("RED")
        lstColor.SelectedIndex = 0
        lstColor.Items.Add("YELLOW")
        lstColor.Items.Add("BLUE")
        lstColor.Items.Add("GREEN")

        'sending byte packet
        bytepacketsend.Add("header_byte", CByte(255)) ' always begin packet with 255 header byte - static, leave alone
        bytepacketsend.Add("panH", CByte(1))
        bytepacketsend.Add("panL", CByte(254))
        bytepacketsend.Add("tiltH", CByte(1))
        bytepacketsend.Add("tiltL", CByte(255))
        bytepacketsend.Add("buttons", CByte(0))
        bytepacketsend.Add("ext_instructions", CByte(8))
        bytepacketsend.Add("checksum", CByte((255 - (CInt(bytepacketsend("panL")) + CInt(bytepacketsend("panH")) + CInt(bytepacketsend("tiltL")) + CInt(bytepacketsend("tiltH")) + CInt(bytepacketsend("buttons")) + CInt(bytepacketsend("ext_instructions"))) Mod 256)))

        'we use these to store a 0 to 1023 value from the scrollbars to make life easy when drawing the crosshairs in pbAim_Paint
        pantilt1023.Add("pan", 512)
        pantilt1023.Add("tilt", 512)

        intBlobSizeMinimum = 20
        tbBlobSizeMin.Value = intBlobSizeMinimum
        txtBlobSizeMin.Text = intBlobSizeMinimum

        'intFactor is the divider we use to translate pixel movement into servo movement.
        'Good settings for the different dynamixels are as follows.
        'AX-12 = 10
        'AX-18 = 7
        intFactor = 10
        tbFactor.Value = intFactor
        txtFactor.Text = intFactor

    End Sub

    Private Sub FormDemo_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' if the port is still open when the form is closing, be sure to default all values and close the port.
        If port.IsOpen Then
            Try
                ResetDefaults()
                port.Close()
            Catch ex As Exception
                Console.WriteLine("Error updating values: " + ex.Message)
            End Try
        End If
    End Sub

    Sub ResetDefaults()
        bytepacketsend("header_byte") = CByte(255)
        bytepacketsend("panH") = CByte(1)
        bytepacketsend("panL") = CByte(255)
        bytepacketsend("tiltH") = CByte(1)
        bytepacketsend("tiltL") = CByte(255)
        bytepacketsend("buttons") = CByte(0)
        bytepacketsend("ext_instructions") = CByte(8)
        bytepacketsend("checksum") = CByte((255 - (CInt(bytepacketsend("panL")) + CInt(bytepacketsend("panH")) + CInt(bytepacketsend("tiltL")) + CInt(bytepacketsend("tiltH")) + CInt(bytepacketsend("buttons")) + CInt(bytepacketsend("ext_instructions"))) Mod 256))

        pantilt1023("pan") = 512
        pantilt1023("tilt") = 512
        hsbPan.Value = 512
        vsbTilt.Value = 512
        pbAim.Refresh()
        UpdateValues()
    End Sub


#End Region


#Region "Serial Communication"


    Private Sub UpdateValues()

        txtServoPan.Text = pantilt1023("pan")
        txtServoTilt.Text = pantilt1023("tilt")

        'Do our checksum at the last moment to make sure it is up to date
        bytepacketsend("checksum") = CByte((255 - (CInt(bytepacketsend("panH")) _
                                                   + CInt(bytepacketsend("panL")) _
                                                   + CInt(bytepacketsend("tiltH")) _
                                                   + CInt(bytepacketsend("tiltL")) _
                                                   + CInt(bytepacketsend("buttons")) _
                                                   + CInt(bytepacketsend("ext_instructions"))) Mod 256))

        If port.IsOpen Then
            Try
                port.Write(bytepacketsend.Values.ToArray(), 0, bytepacketsend.Count())
            Catch ex As Exception
                Console.WriteLine("Error updating values: " + ex.Message)
            End Try
        End If

    End Sub


    Private Sub btnSetPort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSetPort.Click

        Try
            If Not port.IsOpen Then
                ' Setup SerialPort
                port.PortName = cmbPorts.Text
                port.BaudRate = 38400
                port.DataBits = 8
                port.StopBits = System.IO.Ports.StopBits.One
                port.Parity = System.IO.Ports.Parity.None
                port.Open()
                hsbPan.Enabled = True
                vsbTilt.Enabled = True
                btnSetPort.Text = "Close Port"
                UpdateValues()
            Else
                Call ResetDefaults()
                port.Close()
                hsbPan.Enabled = False
                vsbTilt.Enabled = False
                btnSetPort.Text = "Open Port"
            End If
        Catch ex As Exception
            MessageBox.Show("Communications Error: " + ex.Message)
        End Try

    End Sub



#End Region


#Region "Pan & Tilt Controls"

    Private Sub ScrollBars(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles vsbTilt.Scroll, hsbPan.Scroll
        fncCalculateBytes(1023 - hsbPan.Value, 1023 - vsbTilt.Value)
        pbAim.Refresh()
        UpdateValues()
    End Sub

    Private Sub pbAim_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbAim.MouseMove
        ' only move the servos when the left mouse button is clicked,
        ' the port is open, and the location of the mouse is within the correct intBuffer.
        ' the box is 512 x 512 since 1024 x 1024 would be HUGE on the form.
        If e.Button = MouseButtons.Left AndAlso port.IsOpen Then
            If e.Y > 0 AndAlso e.Y <= 512 AndAlso e.X > 0 AndAlso e.X <= 512 Then
                'we do "(e.X * 2) - 1" because the picturebox is half size at 512x512. We need to convert that over to 0 to 1023.
                'we actually are only getting 1 to 1023, but if we can live with that so can you.
                fncCalculateBytes(1023 - ((e.X * 2) - 1), 1023 - ((e.Y * 2) - 1))
                hsbPan.Value = (e.X * 2) - 1
                vsbTilt.Value = (e.Y * 2) - 1
                pbAim.Refresh()
                UpdateValues()
            End If
        End If
    End Sub

    Private Sub fncCalculateBytes(ByVal intPan As Integer, ByVal intTilt As Integer)

        'for the servo horn 1023 is fully rotated left / 0 is fully rotated right
        'thus tilt going up is the # growing toward 1023
        'and panning left (CCW) is the # growing toward 1023

        Dim r As Double
        Dim i As Integer

        'store the 0 - 1023 values
        pantilt1023("pan") = intPan
        pantilt1023("tilt") = intTilt

        'AX & RX Dynamixels have a resolution of 1024 steps (0 to 1023) over 300 degrees
        'But a byte can only store a max # of 255 so we have to do some math magic to split the information into 2 bytes
        i = Math.Truncate(intPan / 256)
        r = intPan - (i * 256)
        bytepacketsend("panL") = CByte(r)
        bytepacketsend("panH") = CByte(i)

        i = Math.Truncate(intTilt / 256)
        r = intTilt - (i * 256)
        bytepacketsend("tiltL") = CByte(r)
        bytepacketsend("tiltH") = CByte(i)

    End Sub

    Private Sub pbAim_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbAim.MouseUp
        pbAim.Refresh()
    End Sub

    Private Sub pbAim_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles pbAim.Paint
        ' create a graphics object on the PictureBox and draw two crosshair lines with it.
        Dim gfx As Graphics = e.Graphics
        Dim myPen As New Pen(Color.Black)

        gfx.DrawLine(myPen, New Point((1023 - pantilt1023("pan")) / 2, 0), New Point((1023 - pantilt1023("pan")) / 2, pbAim.Height)) 'Draw verticle line
        gfx.DrawLine(myPen, New Point(0, (1023 - pantilt1023("tilt")) / 2), New Point(pbAim.Width, (1023 - pantilt1023("tilt")) / 2)) 'Draw horizontal line
    End Sub

#End Region


#Region "RoboRealm Code"

    Private Sub Button_RR_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRR.Click

        If btnRR.Text = "Initialize RoboRealm" Then

            RR.Open("C:\Program Files\RoboRealm\RoboRealm.exe", 6060)
            RR.SetCamera("on")

            If Not RR.Connect("localhost") Then
                'Make sure you have turned on the RoboRealm API!!!!!!! OPTIONS > API SERVER TAB > ACTIVATE CHECKBOX
                MsgBox("OK", MsgBoxStyle.OkOnly, "Could not connect to RoboRealm")
                Exit Sub
            Else
                'Make sure you have preset your camera resolution in RoboRealm 
                '(OPTIONS > VIDEO FORMAT > OUTPUT SIZE > 320 x 240)
                'the demo code is written to work with 320x240 (less pixels = faster response)
                intCameraPixelWidth = RR.GetVariable("IMAGE_WIDTH")
                intCameraPixelHeight = RR.GetVariable("IMAGE_HEIGHT")
                intCenterPan = intCameraPixelWidth / 2
                intCenterTilt = intCameraPixelHeight / 2
                Call Set_Filters()
                tmrTracking.Enabled = True
                btnRR.Text = "Close RoboRealm"
                btnRR.ForeColor = Color.Red
            End If
        Else
            tmrTracking.Enabled = False
            RR.Close()
            chkTracking.Checked = False
            txtBlobSizeMin.BackColor = Color.AliceBlue
            btnRR.Text = "Initialize RoboRealm"
            btnRR.ForeColor = Color.Black
        End If
    End Sub

    Private Sub tmrTracking_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrTracking.Tick

        txtBlobSize.Text = RR.GetVariable("COG_BOX_SIZE")

        If RR.GetVariable("COG_BOX_SIZE") > intBlobSizeMinimum Then

            txtBlobSize.BackColor = Color.LightGreen
            intTempPan = pantilt1023("pan")
            intTempTilt = pantilt1023("tilt")

            'grab the center of gravity of the tracked object
            cogX = RR.GetVariable("COG_X")
            cogY = RR.GetVariable("COG_Y")
            txtCOGPan.Text = cogX
            txtCOGTilt.Text = cogY

            'Calculate the offset from center of camera vision
            intOffsetPan = cogX - intCenterPan
            intOffsetTilt = cogY - intCenterTilt
            txtOffsetPan.Text = intOffsetPan
            txtOffsetTilt.Text = intOffsetTilt

            'apply factor to offset and add to current position
            intTempPan = intTempPan - (intOffsetPan \ intFactor)
            intTempTilt = intTempTilt + (intOffsetTilt \ intFactor)

            'Stay inside 1 to 1023
            If intTempPan > 1023 Then intTempPan = 1023
            If intTempPan < 1 Then intTempPan = 1
            If intTempTilt > 1023 Then intTempTilt = 1023
            If intTempTilt < 1 Then intTempTilt = 1

            If chkTracking.Checked Then
                'write new position to Pan & Tilt
                fncCalculateBytes(intTempPan, intTempTilt)
                'Update form
                pbAim.Refresh()
                hsbPan.Value = 1023 - pantilt1023("pan")
                vsbTilt.Value = 1023 - pantilt1023("tilt")
                'send to microcontroller
                UpdateValues()
            End If
        Else
            txtBlobSize.BackColor = Color.AliceBlue
        End If

    End Sub

    Private Sub lstColor_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstColor.SelectedIndexChanged
        Call Set_Filters()
    End Sub
    Private Sub Set_Filters()

        'The predefined filters will most likely need to be adjusted for your specific envirionment.
        'The settings need to be adjusted for specific cameras, lighting, and object color tones.
        'Simply double click the .robo file to open it in RoboRealm then make adjustments using the 
        'interface and re-save the files.

        Select Case lstColor.SelectedItem.ToString()
            Case "RED"
                RR.LoadProgram(Application.StartupPath & "\robofiles\MS_lifecam_Cinema_track_Red.robo")
            Case "YELLOW"
                RR.LoadProgram(Application.StartupPath & "\robofiles\MS_lifecam_Cinema_track_Yellow.robo")
            Case "BLUE"
                RR.LoadProgram(Application.StartupPath & "\robofiles\MS_lifecam_Cinema_track_Blue.robo")
            Case "GREEN"
                RR.LoadProgram(Application.StartupPath & "\robofiles\MS_lifecam_Cinema_track_Green.robo")
        End Select

    End Sub

    Private Sub tbBlobSizeMin_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbBlobSizeMin.Scroll
        intBlobSizeMinimum = tbBlobSizeMin.Value
        txtBlobSizeMin.Text = intBlobSizeMinimum
    End Sub
    Private Sub tbFactor_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbFactor.Scroll
        intFactor = tbFactor.Value
        txtFactor.Text = intFactor
    End Sub
#End Region




End Class

